/* Include Header files  */

#include "Main.h"

//************************************************
//* Function: Initialize Interrupts              *
//*   - Reset All Interrupr                      *
//*   - Enable HIgh Priority Interrupt           *
//*   - Disable Low Priority Interrupt           *
//************************************************
void InitializeINTs(void) 
{
 	PIR1=0; 						//Clear all Interrupt setting & Flag
	PIR2=0;
	INTCON=0;
	INTCON2=0;
	INTCON3=0;
	PIE1=0;
	PIE2=0;

	RCONbits.IPEN=1;     			//Enable priority interrupts
	INTCONbits.GIEL=1;				// Enable Low Priority Interrupt
	INTCONbits.GIEH=1;				// Enalbe high priority Interrupt
}
//************************************************
//* Function: Initialize USART                   *
//*   - Initialize the RS-232 communication      *
//*   - Disable Interrupr for TxD                *
//*   - Enable Interrupt fro Rxd                 *
//*   - Format: 19200, N, 8, 1                    *
//************************************************
void InitializeUSART(void)
{
	TRISCbits.TRISC7=1;				// Set input for RXD
	TRISCbits.TRISC6=0;				// Set output for TXD
	RCSTAbits.SPEN=1;				// Enable USART Module	
	
	OpenUSART(  USART_TX_INT_OFF 	// Set TXSTA Reg. =0b00100100 
		 	  & USART_RX_INT_ON		// Set RCSTA Reg. =0b10010000 
		 	  & USART_ASYNCH_MODE
		 	  & USART_EIGHT_BIT
		 	  & USART_CONT_RX
		 	  & USART_BRGH_HIGH
		 	  ,51);					// Set SPBRG=51, Baud Rate = 19200 @16MHz

	IPR1bits.RCIP=1;       			// Set Receive of USART are High priority
	PIE1bits.RCIE=1;				// Enable RxD Interrupt
}
//************************************************
//* Initial I2C Master Mode with 7 bits Address  *
//* Clock Speed : 100KHz @16MHz					 *
//************************************************
void Initialize_I2C_Master(void)
{
	TRISCbits.TRISC3=1;
	TRISCbits.TRISC4=1;
	OpenI2C(MASTER,SLEW_ON);
	SSPADD= 39;
	SSPSTATbits.CKE=1;				// SMBus specific inputs
}
//***********************************************
//* Function: Initialize A/D                    *
//*        - Fosc/32 clock source, Tad=2uS      *
//*        - Right justified                    *
//*        - 1 analog channels, 0 references    *
//*        - Channel 0                          *
//*        - Disable A/D interrupt              *
//***********************************************
void InitializeAD(void)
{
	ADCON1 = 0b10000101;   	// ADFM = 1 , PCFG3[3:0] = 0101
	ADCON0 = 0b10001001;	// ADCS[2:0] = 010, Tad=16MHz/32=2uS
							// A/D ON , Select Channel = AN1
	PIE1bits.ADIE=0;		// Disable A/D interrupt
}

//************************************************
//*        Function: Initialize Timer1           *
//************************************************
void InitializeTMR1(void)
{
	T1CON=0b10001111;			// Initialize Timer1 for using External 32.768KHz   
	IPR1bits.TMR1IP=0;			// Set Timer1 for Low Priority Interrupt
	PIR1bits.TMR1IF=0;			// Clear Interrupt flag of Timer 1
	
	TMR1H = (0xFFFF-Timer_Count)/256;	
	TMR1L = (0xFFFF-Timer_Count)%256;	// Set for 0.1 Second Interrupt Time
	
	PIE1bits.TMR1IE=1;			// Enable Timer1 Interrupt
}

//************************************************
//*        Function: Initialize Timer2           *
//************************************************
void InitializeTMR2(void)
{
		OpenTimer2 (TIMER_INT_ON		// Turn On the Timer2 with Interrupt
					& T2_PS_1_4
					& T2_POST_1_15);

		PR2 = 0xFF;						
		IPR1bits.TMR2IP=1;				// Set Timer2 for The Low Priority Interrupt
}

//************************************************
//*        Function: Initialize PWM1&2           *
//************************************************
void InitializePWM(void)
{
	OpenPWM1(0xff); 					// Open PWM1 for Buzzer
	OpenPWM2(0xFF);						// Open PWM2 for FAN
	SetDCPWM1(0x3FF/2);					// Buzzer out frequency:
										// PWM = (PR2+1)*4*(1/16MHz)(Prescal)
										// 3.9KKz = 256 *4*0.0625uS * 4 = 256uS
	TRISCbits.TRISC2=1;					// Turn off the PWM1 output (Buzzer)
	TRISBbits.TRISB3=0; 				// Turn on the PWM2 output	(FAN)
}
//************************************************
//*        Function: Initialize PORT             *
//************************************************
void InitializePORT(void)
{
	ADCON1=0b00000101;    		// Enable AN0 & AN1 with External Vref
	PORTA=0x00;
	TRISA=0b11011011;			// RA0 & RA1 as A/D input,RA4 as SW2 input
								// RA2 as LCD-E control, RA5 as CS Control  for SPI
	PORTB=0x00;
	TRISB=0b11001001;			// RB[6:7] as ICD2, 
								// RB2 as CANTX , RB3 as CANRX
	PORTC=0x00;
	
	PORTD=0x00;
}
