//***********************************************
//*												*
//*           	  Exercise 7-3					*
//*												*
//***********************************************
/* Include Header files  */


#include <p18cxxx.h>
#include <adc.h>
#include <pwm.h>
#include <timers.h>
#include <delays.h>
#include <usart.h>
#include <stdlib.h>
#include <ctype.h>
#include <i2c.h>
#include "p18F_LCD.h"
#include "I2C_EEP_SUBS_24LC32A.h"

	#pragma config	OSC=HS
	#pragma config 	PWRT=OFF
	#pragma config 	BOREN=OFF, BORV=1
	#pragma config 	WDT=OFF
	#pragma config  CCP2MX=PORTC
	#pragma config  STVREN=OFF, LVP=OFF, DEBUG=OFF
	#pragma config  CPD=OFF

/* Declare the Function Prototype */
void InitializeAD(void);
void InitializePWM1(void);
void InitializeTMR2(void);
void InitializePORT(void);
void InitializeUSART(void);
void InitializeINTs(void);
void Initialize_I2C_Master(void);
void isr_high_direct(void);
void isr_high(void);
void Display_Terminal(const rom far char *data);
void Send_CR_LF(void);
 
#define NULL 0x00
#define Min_Langth 4

/* define the LCD_MSG1[] in program memory at 0x1000 */
//***************************************************** 
#pragma romdata My_romdata=0x1000
const rom far unsigned char LCD_MSG1[]="Waiting Data Now!";
const rom far unsigned char LCD_MSG2[]="-Display EEPROM- ";

const rom far char Disp_Msg1[]="Please Enter your Number :";
const rom far char Disp_Msg2[]="EEPROM WRITE is completed!"; 
const rom far char Disp_Msg3[]="\aError! Please enter again:";
const rom far char Disp_Msg4[]="\aYou must get 4 Digials at least! ";
const rom far char Disp_Msg5[]="Completed to save your Number!";
#pragma romdata

/* define following variable in data memory at Access Bank */
//*********************************************************** 
#pragma udata access My_RAM_2

near unsigned char f;
near unsigned int j;

union					
{
int l;
char i[2];
} near EE_Read_Data;

#pragma udata

unsigned char Rec_Data;
unsigned int EE_Addr;

//***********************************************************
/*														   */
/* 						Main 	Program					   */
/*														   */
//***********************************************************

void main(void)
{

	InitializePORT( );
// 	InitializePWM1( );
// 	InitializeAD( );		
// 	InitializeTMR2( );
	OpenLCD( );
	Initialize_I2C_Master( );
	InitializeINTs( );							
	InitializeUSART( );

	Display_Terminal(Disp_Msg1);					// Send Disp_Msg1 to Hyper-Terminal

 	LCD_Set_Cursor(0,0);						// Put LCD Cursor on Line 1
	putrsLCD(LCD_MSG1);

	Rec_Data=0x00;									// Clear Receive data, it is from RS-232
	EE_Addr=0;										// Set EEPROM start address = 0

	do
	{
		while(Rec_Data==0x00);									// New RS-232 data received ? No, Loop here.
		

			if (isdigit(Rec_Data ))								// Received data is a decimal digit
				{
					f=EEByteWrite_L (0xA0,EE_Addr,Rec_Data);		// Write Receive Data into EEPROM
					f=EEAckPolling(0xA0);						// Polling the Acknowledge
					EE_Addr++;									// Point to next address of EEPROM 
					Rec_Data=0x00;								
					Display_Terminal(Disp_Msg2);				// Send Disp_Msg2 to Hyper-Terminal
				}
				else if (Rec_Data!=0x0D) 						// Press "Enter" key or alphabetic character
					{
						Display_Terminal(Disp_Msg3);			// alphabetic, display error message to Hyper-Terminal
						Rec_Data=0x00;
					}
					else if (EE_Addr<Min_Langth) 				// "Enter" key, need to check how many digit stored
						{
							Display_Terminal(Disp_Msg4);		// send error message to Hyper_Terminal
							Rec_Data=0x00;
						}
		
		
	} while ((EE_Addr<10) && ((Rec_Data!=0x0d)||(EE_Addr<Min_Langth)));  // Fully check

	{															// Write NULL into EEPROM for a string format
		Rec_Data=NULL;
		f=EEByteWrite_L (0xA0,EE_Addr,Rec_Data);
		f=EEAckPolling(0xA0);
		Display_Terminal(Disp_Msg5);
	}

	LCD_Set_Cursor(0,0);										// Put LCD Cursor on Line 1
	putrsLCD(LCD_MSG2);							

	LCD_Set_Cursor(1,0);											// Put LCD Cursor on Line 2
	j=0;

	while(1)													// loop here until read the NULL from EEPROM
	{
		EE_Read_Data.l = EERandomRead_L(0xA0,j);					// Read data from EEPROM

		if (EE_Read_Data.i[0]!=NULL)							// check NULL data?
			{
				WriteDataLCD(EE_Read_Data.i[0]);				// No, display the data to LCD
				j++;											// Point to next address of EEPROM
			}
		else	break;											//Detected NUll, Break out the loop
	}



	while(1);													// End of process, loop
}
//***********************************************
//*   Send a ASCII string to Terminal           *
//***********************************************
void Display_Terminal(const rom far char *data)	
{	
		Send_CR_LF();
		putrsUSART(data);
		Delay10KTCYx(10);
}
void Send_CR_LF(void)
{
		while (BusyUSART());
		WriteUSART('\n');
		while (BusyUSART());
		WriteUSART('\r');
}
//************************************************
//*       #pragma Interrupt Declarations         *
//************************************************
//************************************************
//* Function: isr_high_direct                    *
//*   - Direct execution to the actual           *
//*     high-priority interrupt code.            *
//************************************************
#pragma code isrhighcode = 0x0008

void isr_high_direct(void)	
{  								
     _asm 		//begin in-line assembly
     goto isr_high	//go to isr_high function
     _endasm 		//end in-line assembly
}
#pragma code

//************************************************
//* Function: isr_high(void)                     *     
//*   High priority interrupt will               *
//*   - Received a serial data from RS-232       *
//*     Save the received data to buffer Rec_Data*                  
//************************************************
#pragma interrupt isr_high 

void isr_high(void)
{
	PIR1bits.RCIF=0;					// Clear RxD Flag
	Rec_Data=ReadUSART();				// Get RS-232 data
 	PORTD=Rec_Data;						// Display to LEDs
}								
#pragma code


